import pandas as pd
import pytest

from pyplattsapi import world_oil_supply


@pytest.mark.parametrize(
    "filter, field, groupBy",
    [
        (
            'countryname:"Italy" and supplyTypeName:"Crude" and productionTypeName:"Production"',
            None,
            None,
        ),
        (
            'countryname:"Saudi Arabia" and supplyTypeName:"Crude" and productionTypeName:"Production"',
            "sum(value)",
            "countryName, year, month",
        )
    ],
)
def test_get_production(filter: str, field: str, groupBy: str):
    res = world_oil_supply.get_production(filter=filter, field=field, groupBy=groupBy)
    assert isinstance(res, pd.DataFrame)


@pytest.mark.parametrize(
    "metadata_endpoint, filter, field, groupBy",
    [
        (
            "grade-elements",
            'gradeElementName:"Zarzaitine"',
            None,
            None,
        ),
        (
            "countries",
            None,
            None,
            None,
        ),

    ],
)
def test_get_metadata(metadata_endpoint:str, filter: str, field: str, groupBy: str):
    res = world_oil_supply.get_metadata(metadata_endpoint=metadata_endpoint, filter=filter, field=field, groupBy=groupBy)
    assert isinstance(res, pd.DataFrame)


country_map = {
    'Albania': 'AL',
    "Algeria": "DZ",
    'Angola': 'AO',
    'Argentina': 'AR',
    'Austria': 'AT',
    'Australia': 'AU',
    'Azerbaijan': 'AZ',
    'Barbados': 'BB',
    'Bangladesh': 'BD',
    'Bulgaria': 'BG',
    'Bahrain': 'BH',
    'Brunei': 'BN',
    'Bolivia': 'BO',
    'Brazil': 'BR',
    'Belarus': 'BY',
    'Canada': 'CA',
    'Congo, dem. Rep.': 'CD',
    'Congo, Republic of.': 'CG',
    'Chile': 'CL',
    'Cameroon': 'CM',
    'China': 'CN',
    'Colombia': 'CO',
    'Cuba': 'CU',
    'Czech Republic': 'CZ',
    'Germany': 'DE',
    'Denmark': 'DK',
    'Algeria': 'DZ',
    'Ecuador': 'EC',
    'Estonia': 'EE',
    'Egypt': 'EG',
    'Spain': 'ES',
    'France': 'FR',
    'Gabon': 'GA',
    'United Kingdom': 'GB',
    'Ghana': 'GH',
    'Equatorial Guinea': 'GQ',
    'Greece': 'GR',
    'Guatemala': 'GT',
    'Guyana': 'GY',
    'Croatia': 'HR',
    'Hungary': 'HU',
    'Indonesia': 'ID',
    'India': 'IN',
    'Iraq': 'IQ',
    'Iran': 'IR',
    'Italy': 'IT',
    'Jordan': 'JO',
    'Japan': 'JP',
    'Kenya': 'KE',
    'Kyrgyzstan': 'KG',
    'Cambodia': 'KH',
    'Kuwait': 'KW',
    'Kazakhstan': 'KZ',
    'Lithuania': 'LT',
    'Libya': 'LY',
    'Morocco': 'MA',
    'Madagascar': 'MG',
    'Myanmar': 'MM',
    'Mauritania': 'MR',
    'Mexico': 'MX',
    'Malaysia': 'MY',
    'Niger': 'NE',
    'Nigeria': 'NG',
    'Netherlands': 'NL',
    'Norway': 'NO',
    'Neutral Zone': None,
    'United Arab Emirates': 'AE',
    'New Zealand': 'NZ',
    'Oman': 'OM',
    'Peru': 'PE',
    'Papua New Guinea': 'PG',
    'Philippines': 'PH',
    'Pakistan': 'PK',
    'Poland': 'PL',
    'Qatar': 'QA',
    'Romania': 'RO',
    'Serbia': 'RS',
    'Russia': 'RU',
    'Saudi Arabia': 'SA',
    'Sudan': 'SD',
    'Suriname': 'SR',
    'Syria': 'SY',
    'Chad': 'TD',
    'Thailand': 'TH',
    'Tajikistan': 'TJ',
    'East Timor': 'TL',
    'Turkmenistan': 'TM',
    'Tunisia': 'TN',
    'Turkey': 'TR',
    'Trinidad & Tobago': 'TT',
    'Taiwan': 'TW',
    'Ukraine': 'UA',
    'USA': 'US',
    'United States': 'US',
    'Georgia': 'GE',
    'Uzbekistan': 'UZ',
    'Venezuela': 'VE',
    'Vietnam': 'VN',
    'Yemen': 'YE',
    'South Africa': 'ZA',

}



import pandas as pd
import pytest

from pyplattsapi import refining_margins_crude_arbitrage as m


@pytest.mark.parametrize(
    "filter, FrequencyId",
    [
        (
            'marginId:1',
            1,
        ),
        (
            'marginId:1',
            2,
        )
    ],
)
def test_margins_data(filter: str, FrequencyId: int):#, groupBy: str):
    res = m.margins_data(filter=filter, FrequencyId=FrequencyId)
    assert isinstance(res, pd.DataFrame)


import pytest
from pyplattsapi import plattsapicore


def test_get_access_token():
    res = plattsapicore.get_access_token(api_dataset="REFINING MARGINS AND CRUDE ARBITRAGE")
    assert res is not None


def test_build_header():
    res = plattsapicore.build_header(api_dataset="WORLD OIL SUPPLY")
    assert "Authorization" in res


#
#
# def test_get_capacity_changes_by_country():
#     res = pyplattsapi.world_refinery_database.getCountryCapacityChangesTimeSeries('China')
#     assert 'Mbdc' in res.columns
#     assert 'Mmtcd' in res.columns
#     assert 'Mmtcy' in res.columns
#
#
# def test_get_yearly_runs_by_country():
#     res = pyplattsapi.world_refinery_database.getCountryYearlyRunsTimeSeries('China')
#     assert 'Mbdc' in res.columns
#     assert 'Mmtcd' in res.columns
#     assert 'Mmtcy' in res.columns
#
#
# def test_get_yearly_margins_by_type():
#     res = pyplattsapi.world_refinery_database.getMarginsbyType('Dated Brent NWE Cracking')
#     assert 'Date' in res.columns
#     assert 'Margin' in res.columns
#
#
# def test_us_inventories_by_product():
#     res = pyplattsapi.oil_inventory.getUSInventoriesByProduct('Central Atlantic Distillate')
#     assert 'Date' in res.columns
#     assert 'Volume' in res.columns
#
#
# def test_us_imports_by_product():
#     res = pyplattsapi.oil_inventory.getUSImportsByProduct('Fuel Oil')
#     assert 'ArrivalDate' in res.columns
#     assert 'Volume' in res.columns
#
#
# def test_us_crude_exports():
#     res = pyplattsapi.oil_inventory.getUSCrudeExports()
#     assert 'EIAWeekEnding' in res.columns
#     assert 'ForecastType' in res.columns
#     assert 'Volume_MBD' in res.columns



import pandas as pd
import pytest

from pyplattsapi import world_refinery_database


@pytest.mark.parametrize(
    "filter, field, groupBy",
    [
        (
            'RefineryId eq 1',
            None,
            None,
        ),
    ],
)
def test_get_runs(filter: str, field: str, groupBy: str):
    res = world_refinery_database.get_runs(filter=filter, field=field, groupBy=groupBy)
    assert isinstance(res, pd.DataFrame)



import pandas as pd

from pyplattsapi import plattsapicore

# https://developer.platts.com/servicecatalog#/WorldOilSupply(Beta)/v2/

api_name = "WORLD OIL SUPPLY"
production_api = f"{plattsapicore.api_url}/wos/v2/production"
metadata_api = f"{plattsapicore.api_url}/wos/v2/metadata_endpoint"


def get_production(filter, field, groupBy, scenarioTermId: int = 2):
    params = {
        "filter": filter,
        "scenarioTermId": scenarioTermId,
        "field": field,
        "pageSize": 1000,
        "groupBy": groupBy,
        "page": 1,
    }
    res = plattsapicore.generic_api_call(api=production_api, api_name=api_name, params=params)

    if 'month' in res.columns and 'year' in res.columns:
        res.index = res.apply(lambda x: pd.to_datetime(f"{x.year}-{x.month}-1"), 1)
        res.index.name = "date"
    return res


def get_metadata(metadata_endpoint:str, filter:str=None, field:str=None, groupBy:str=None) -> pd.DataFrame:
    params = {
        "filter": filter,
        "field": field,
        "pageSize": 1000,
        "groupBy": groupBy,
        "page": 1,
    }
    url = metadata_api.replace('metadata_endpoint', metadata_endpoint)
    res = plattsapicore.generic_api_call(api=url, api_name=api_name, params=params)
    return res




import os
import time
from functools import lru_cache
from urllib.error import HTTPError

import pandas as pd
import requests
from cachetools.func import ttl_cache

api_url = "https://api.platts.com"
auth_url = f"{api_url}/auth/api"


@lru_cache(maxsize=20)
def extract_credential(key: str):
    credentials = os.getenv("PLATTS_API_CREDENTIALS")
    tokens = credentials.split(";")
    token = [x for x in tokens if x.startswith(key)]
    if len(token) > 0:
        s = token[0].split("=")
        return s[1]


def get_access_token(api_dataset):
    """
    Given an api_dataset (eg OIL INVENTORY), get an access token from the Auth API
    :param api_dataset:
    :return:
    """
    data = {
        "username": extract_credential("username"),
        "password": extract_credential("password"),
    }
    headers = {"appkey": extract_credential(api_dataset)}
    token_request_sup = requests.post(auth_url, headers=headers, data=data)
    req_dic_sup = token_request_sup.json()
    access_token_sup = req_dic_sup["access_token"]
    return access_token_sup


@ttl_cache(ttl=10 * 60)
def build_header(api_dataset):
    header = {
        "accept": "application/json",
        "appkey": extract_credential(api_dataset),
        "Authorization": f"Bearer {get_access_token(api_dataset)}",
    }
    return header


def generic_api_call_helper(api: str, api_name: str, params: dict, page: int = None):
    if page is not None:
        params['page'] = page
    response = requests.get(
        url=api, headers=build_header(api_name), params=params
    )
    if response.status_code == 200:
        return response.json()
    else:
        raise HTTPError(url=api, code=response.status_code, msg=response.json()['cause'], hdrs=None, fp=None)


def generic_api_call(api: str, api_name: str, params: dict):
    # make call for page 1
    r = generic_api_call_helper(api, api_name, params, page=1)
    max_page = int(r["metadata"]["count"] / 1000)
    res = pd.concat([pd.Series(x) for x in r["results"]], axis=1)
    # make call for pages 2 -> n (if needed as determined by max_page)
    for page in range(2, max_page + 2):
        time.sleep(0.55) # 2 calls per second allowed
        r = generic_api_call_helper(api, api_name, params, page=page)
        d = pd.concat([pd.Series(x) for x in r["results"]], axis=1)
        res = pd.concat([d, res], axis=1)

    res = res.T
    return res

def generic_odata_call(api: str, api_name: str, params: dict):
    # make call for page 1
    r = generic_api_call_helper(api, api_name, params)
    res = pd.concat([pd.Series(x) for x in r["value"]], axis=1)
    nextPage = r['@odata.nextLink'] if '@odata.nextLink' in r else None
    while nextPage:
        r = generic_api_call_helper(r['@odata.nextLink'], api_name=api_name, params=None)
        d = pd.concat([pd.Series(x) for x in r["value"]], axis=1)
        res = pd.concat([d, res], axis=1)
        nextPage = r['@odata.nextLink'] if '@odata.nextLink' in r else None

    res = res.T
    return res


import time

import pandas as pd
import requests

from pyplattsapi import plattsapicore

# https://developer.platts.com/servicecatalog#/ArbFlow(Beta)/v1/


api_name = "REFINING MARGINS AND CRUDE ARBITRAGE"
production_api = f"{plattsapicore.api_url}/arbflow/data/v1/margins"


def margins_data(filter:str, FrequencyId:int=1):
    margins_data_api = f"{plattsapicore.api_url}/arbflow/data/v1/margins-data"
    params = {
        'pageSize': 1000,
        'FrequencyId': FrequencyId,
        'filter': filter,
        'sort' : 'marginDate:desc'
    }
    res = plattsapicore.generic_api_call(margins_data_api, api_name=api_name, params=params)
    res.index = res['marginDate']
    return res


def margins_catalog():
    margins_catalog_api = f"{plattsapicore.api_url}/arbflow/data/v1/margins-catalog"
    params = {'pageSize': 1000}
    response = requests.get(
        url=margins_catalog_api, headers=plattsapicore.build_header(api_name), params=params
    )
    d = response.json()
    d['results']
    df = pd.concat([pd.Series(x) for x in d["results"]], axis=1).T
    return df


def margins_metadata():
    margins_metadata_api = f"{plattsapicore.api_url}/arbflow/data/v1/margins"
    params = {'pageSize': 1000}
    response = requests.get(
        url=margins_metadata_api, headers=plattsapicore.build_header(api_name), params=params
    )
    d = response.json()
    d['results']
    df = pd.concat([pd.Series(x) for x in d["results"]], axis=1).T
    return df


import pandas as pd

from pyplattsapi import plattsapicore

api_name = "WORLD REFINERY DATABASE"
runs_api = f"{plattsapicore.api_url}/odata/refinery-data/v2/Runs"


def get_runs(filter:str, field:str=None, groupBy:str=None):
    params = {
        "$filter": filter,
        "pageSize": 1000,
        "groupBy": groupBy,

    }
    res = plattsapicore.generic_odata_call(api=runs_api, api_name=api_name, params=params)

    qmap = { 1: 1, 2: 4, 3: 7, 4: 10 }
    res.index = res.apply(lambda x: pd.to_datetime(f"{x.Year}-{qmap.get(x.Quarter)}-1"), 1)
    res.index.name = "date"
    return res



#
# def getMarginsbyType(type: str):
#     Historical_data_URL = f"https://api.platts.com/odata/refinery-data/v2/Margins?&pageSize=1000&$expand=*"
#     df5 = pd.DataFrame()
#     while Historical_data_URL != "NaN":
#         time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
#         data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_ref)
#         data = data_request.json()
#         df2 = pd.json_normalize(data).reset_index(drop=True)
#         x = df2["value"].iloc[0]
#         df3 = pd.json_normalize(x).reset_index(drop=True)
#         df3 = df3.drop_duplicates()
#         df5 = df5.append(df3, ignore_index=False)
#         try:
#             Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
#         except:
#             Historical_data_URL = "NaN"
#             continue
#     df5 = df5[df5["MarginType.Name"] == type]
#     df5 = df5.reset_index()
#     return df5
#
#
# def getCountryCapacityChangesTimeSeries(country: str):
#     Historical_data_URL = f"https://api.platts.com/odata/refinery-data/v2/capacity?$select=*&$expand=*&pageSize=1000&$filter=Refinery/Country/Name eq '{country}'"
#     df5 = pd.DataFrame()
#     while Historical_data_URL != "NaN":
#         time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
#         data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_sup)
#         data = data_request.json()
#         df2 = pd.json_normalize(data).reset_index(drop=True)
#         x = df2["value"].iloc[0]
#         df3 = pd.json_normalize(x).reset_index(drop=True)
#         df3 = df3.drop_duplicates()
#         df3["Date"] = df3[["Year", "Quarter"]].apply(
#             lambda row: "-Q".join(row.values.astype(str)), axis=1
#         )
#         df4 = df3[["Refinery.Name", "Mbcd", "Mmtcd", "Mmtcy", "Date"]]
#         df4 = df4.groupby(["Date", "Refinery.Name"]).sum()
#         df4 = df4.reset_index()
#         df4["Date"] = pd.to_datetime(df4["Date"])
#         df5 = df5.append(df4, ignore_index=False)
#         try:
#             Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
#         except:
#             Historical_data_URL = "NaN"
#             continue
#     df4.columns = ["Date", "Refinery", "Mbdc", "Mmtcd", "Mmtcy"]
#     df5 = df4.groupby(["Date"]).sum()
#     df6 = df5.reset_index()
#     return df6



import time

import pandas as pd
import requests

from plattsapi.plattsapicore import Headers_inv


def getUSInventoriesByProduct(product: str):
    Historical_data_URL = f"https://api.platts.com/odata/inventory-data/v2/USWeeklyCrudeAndRefinedProductInventoryForecasts?$select=*&pageSize=1000"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_inv)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df5 = df5.append(df3, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df5 = df5[df5["ProductName"] == product]
    df5 = df5.reset_index()
    return df5


def getUSImportsByProduct(product: str):
    Historical_data_URL = f"https://api.platts.com/odata/inventory-data/v2/USPetroleumImportTradeStatistics?$select=*&pageSize=1000"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_inv)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df5 = df5.append(df3, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df5 = df5[df5["ProductName"] == product]
    df5 = df5.groupby(["ArrivalDate"]).sum()
    df5 = df5.reset_index()
    return df5


def getUSCrudeExports():
    Historical_data_URL = f"https://api.platts.com/odata/inventory-data/v2/USCrudeExports?$select=*&pageSize=1000"
    df5 = pd.DataFrame()
    while Historical_data_URL != "NaN":
        time.sleep(1)  # api can only accept 2 requests per second and 5000 per day
        data_request = requests.get(url=f"{Historical_data_URL}", headers=Headers_inv)
        data = data_request.json()
        df2 = pd.json_normalize(data).reset_index(drop=True)
        x = df2["value"].iloc[0]
        df3 = pd.json_normalize(x).reset_index(drop=True)
        df3 = df3.drop_duplicates()
        df5 = df5.append(df3, ignore_index=False)
        try:
            Historical_data_URL = df2[f"@odata.nextLink"].iloc[0]
        except:
            Historical_data_URL = "NaN"
            continue
    df5 = df5.reset_index()
    return df5
